@@ -2,13 +2,12 @@ package Time::Piece;
 
 use strict;
 
-require DynaLoader;
+use XSLoader ();
 use Time::Seconds;
 use Carp;
 use Time::Local;
+use Scalar::Util qw/ blessed /;
 
-our @ISA = qw(DynaLoader);
- 
 use Exporter ();
 
 our @EXPORT = qw(
@@ -20,18 +19,20 @@ our %EXPORT_TAGS = (
     ':override' => 'internal',
     );
 
-our $VERSION = '1.32';
+our $VERSION = '1.37';
 
-bootstrap Time::Piece $VERSION;
+XSLoader::load( 'Time::Piece', $VERSION );
 
 my $DATE_SEP = '-';
 my $TIME_SEP = ':';
+my $DATE_FORMAT = '%a, %d %b %Y %H:%M:%S %Z';
 my @MON_LIST = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
 my @FULLMON_LIST = qw(January February March April May June July
                       August September October November December);
 my @DAY_LIST = qw(Sun Mon Tue Wed Thu Fri Sat);
 my @FULLDAY_LIST = qw(Sunday Monday Tuesday Wednesday Thursday Friday Saturday);
 my $IS_WIN32 = ($^O =~ /Win32/);
+my $IS_LINUX = ($^O =~ /linux/i);
 
 my $LOCALE;
 
@@ -65,13 +66,30 @@ sub gmtime {
     $class->_mktime($time, 0);
 }
 
+
+# Check if the supplied param is either a normal array (as returned from
+# localtime in list context) or a Time::Piece-like wrapper around one.
+#
+# We need to differentiate between an array ref that we can interrogate and
+# other blessed objects (like overloaded values).
+sub _is_time_struct {
+    return 1 if ref($_[1]) eq 'ARRAY';
+    return 1 if blessed($_[1]) && $_[1]->isa('Time::Piece');
+
+    return 0;
+}
+
+
 sub new {
     my $class = shift;
     my ($time) = @_;
 
     my $self;
 
-    if (defined($time)) {
+    if ($class->_is_time_struct($time)) {
+        $self = $time->[c_islocal] ? $class->localtime($time) : $class->gmtime($time);
+    }
+    elsif (defined($time)) {
         $self = $class->localtime($time);
     }
     elsif (ref($class) && $class->isa(__PACKAGE__)) {
@@ -99,20 +117,16 @@ sub parse {
         @components = shift =~ /(\d+)$DATE_SEP(\d+)$DATE_SEP(\d+)(?:(?:T|\s+)(\d+)$TIME_SEP(\d+)(?:$TIME_SEP(\d+)))/;
         @components = reverse(@components[0..5]);
     }
-    return $class->new(_strftime("%s", timelocal(@components)));
+    return $class->new( timelocal(@components ));
 }
 
 sub _mktime {
     my ($class, $time, $islocal) = @_;
 
-    $class = eval { (ref $class) && (ref $class)->isa('Time::Piece') }
-           ? ref $class
-           : $class;
-    if (ref($time)) {
-        my @tm_parts = (@{$time}[c_sec .. c_mon], $time->[c_year]+1900);
-        $time->[c_epoch] = $islocal ? timelocal(@tm_parts) : timegm(@tm_parts);
+    $class = blessed($class) || $class;
 
-        return wantarray ? @$time : bless [@$time[0..9], $islocal], $class;
+    if ($class->_is_time_struct($time)) {
+        return wantarray ? @$time : bless [@$time[0..8], undef, $islocal], $class;
     }
     _tzset();
     my @time = $islocal ?
@@ -284,7 +298,12 @@ sub yday {
 
 sub isdst {
     my $time = shift;
-    $time->[c_isdst];
+    return 0 unless $time->[c_islocal];
+    # Calculate dst based on current TZ
+    if ( $time->[c_isdst] == -1 ) {
+        $time->[c_isdst] = ( CORE::localtime( $time->epoch ) )[-1];
+    }
+    return $time->[c_isdst];
 }
 
 *daylight_savings = \&isdst;
@@ -460,8 +479,7 @@ sub month_last_day {
     return $MON_LAST[$_mon] + ($_mon == 1 ? _is_leap_year($year) : 0);
 }
 
-my $trans_map_common = {
-
+my $strftime_trans_map = {
     'c' => sub {
         my ( $format ) = @_;
         if($LOCALE->{PM} && $LOCALE->{AM}){
@@ -472,6 +490,43 @@ my $trans_map_common = {
         }
         return $format;
     },
+    'e' => sub {
+        my ( $format, $time ) = @_;
+        my $day = sprintf( "%2d", $time->[c_mday] );
+        $format =~ s/%e/$day/ if $IS_WIN32;
+        return $format;
+    },
+    'D' => sub {
+        my ( $format ) = @_;
+        $format =~ s/%D/%m\/%d\/%y/;
+        return $format;
+    },
+    'F' => sub {
+        my ( $format ) = @_;
+        $format =~ s/%F/%Y-%m-%d/;
+        return $format;
+    },
+    'k' => sub {
+        my ( $format, $time ) = @_;
+        my $hr = sprintf( "%2d", $time->[c_hour] );
+        $format =~ s/%k/$hr/ if $IS_WIN32;
+        return $format;
+    },
+    'l' => sub {
+        my ( $format, $time ) = @_;
+        if ($IS_WIN32) {
+            my $hr = $time->[c_hour] > 12 ? $time->[c_hour] - 12 : $time->[c_hour];
+            $hr = sprintf( "%2d", $hr );
+            $format =~ s/%l/$hr/ if $IS_WIN32;
+        }
+        return $format;
+    },
+    'P' => sub {
+        my ( $format ) = @_;
+        # %P seems to be linux only
+        $format =~ s/%P/%p/ unless $IS_LINUX;
+        return $format;
+    },
     'r' => sub {
         my ( $format ) = @_;
         if($LOCALE->{PM} && $LOCALE->{AM}){
@@ -482,38 +537,8 @@ my $trans_map_common = {
         }
         return $format;
     },
-    'X' => sub {
-        my ( $format ) = @_;
-        if($LOCALE->{PM} && $LOCALE->{AM}){
-            $format =~ s/%X/%I:%M:%S %p/;
-        }
-        else{
-            $format =~ s/%X/%H:%M:%S/;
-        }
-        return $format;
-    },
-};
-
-my $strftime_trans_map = {
-    %{$trans_map_common},
-
-    'e' => sub {
-        my ( $format, $time ) = @_;
-        $format =~ s/%e/%d/ if $IS_WIN32;
-        return $format;
-    },
-    'D' => sub {
-        my ( $format, $time ) = @_;
-        $format =~ s/%D/%m\/%d\/%y/;
-        return $format;
-    },
-    'F' => sub {
-        my ( $format, $time ) = @_;
-        $format =~ s/%F/%Y-%m-%d/;
-        return $format;
-    },
     'R' => sub {
-        my ( $format, $time ) = @_;
+        my ( $format ) = @_;
         $format =~ s/%R/%H:%M/;
         return $format;
     },
@@ -521,16 +546,17 @@ my $strftime_trans_map = {
         #%s not portable if time parts are from gmtime since %s will
         #cause a call to native mktime (and thus uses local TZ)
         my ( $format, $time ) = @_;
-        $format =~ s/%s/$time->[c_epoch]/;
+        my $e = $time->epoch();
+        $format =~ s/%s/$e/;
         return $format;
     },
     'T' => sub {
-        my ( $format, $time ) = @_;
+        my ( $format ) = @_;
         $format =~ s/%T/%H:%M:%S/ if $IS_WIN32;
         return $format;
     },
     'u' => sub {
-        my ( $format, $time ) = @_;
+        my ( $format ) = @_;
         $format =~ s/%u/%w/ if $IS_WIN32;
         return $format;
     },
@@ -541,10 +567,20 @@ my $strftime_trans_map = {
         return $format;
     },
     'x' => sub {
-        my ( $format, $time ) = @_;
+        my ( $format ) = @_;
         $format =~ s/%x/%a %d %b %Y/;
         return $format;
     },
+    'X' => sub {
+        my ( $format ) = @_;
+        if($LOCALE->{PM} && $LOCALE->{AM}){
+            $format =~ s/%X/%I:%M:%S %p/;
+        }
+        else{
+            $format =~ s/%X/%H:%M:%S/;
+        }
+        return $format;
+    },
     'z' => sub {    #%[zZ] not portable if time parts are from gmtime
         my ( $format, $time ) = @_;
         $format =~ s/%z/+0000/ if not $time->[c_islocal];
@@ -559,7 +595,7 @@ my $strftime_trans_map = {
 
 sub strftime {
     my $time = shift;
-    my $format = @_ ? shift(@_) : '%a, %d %b %Y %H:%M:%S %Z';
+    my $format = @_ ? shift(@_) : $DATE_FORMAT;
     $format = _translate_format($format, $strftime_trans_map, $time);
 
     return $format unless $format =~ /%/; #if translate removes everything
@@ -567,20 +603,75 @@ sub strftime {
     return _strftime($format, $time->epoch, $time->[c_islocal]);
 }
 
-my $strptime_trans_map = {
-    %{$trans_map_common},
-};
-
 sub strptime {
-    my $time = shift;
+    my $time   = shift;
     my $string = shift;
-    my $format = @_ ? shift(@_) : "%a, %d %b %Y %H:%M:%S %Z";
-    my $islocal = (ref($time) ? $time->[c_islocal] : 0);
-    my $locales = $LOCALE || &Time::Piece::_default_locale();
-    $format = _translate_format($format, $strptime_trans_map);
-    my @vals = _strptime($string, $format, $islocal, $locales);
-#    warn(sprintf("got vals: %d-%d-%d %d:%d:%d\n", reverse(@vals[c_sec..c_year])));
-    return scalar $time->_mktime(\@vals, $islocal);
+    my $format;
+    my $opts;
+
+    if ( @_ >= 2 && blessed( $_[1] ) && $_[1]->isa('Time::Piece') ) {
+        # $string, $format, $time_piece_object
+        $format = shift;
+        $opts   = { defaults => shift };
+    } elsif ( @_ && blessed( $_[0] ) && $_[0]->isa('Time::Piece') ) {
+        # $string, $time_piece_object
+        $opts   = { defaults => shift };
+        $format = $DATE_FORMAT;
+    } elsif ( @_ >= 2 && ref( $_[1] ) eq 'HASH' ) {
+        # $string, $format, {options => ...}
+        $format = shift;
+        $opts   = shift;
+    } elsif ( @_ && ref( $_[0] ) eq 'HASH' ) {
+        # $string, {options => ...}
+        $opts   = shift;
+        $format = @_ ? shift : $DATE_FORMAT;
+    } else {
+        $format = @_ ? shift : $DATE_FORMAT;
+    }
+
+    my $islocal  = ( ref($time) ? $time->[c_islocal] : 0 );
+    my $locales  = $LOCALE || &Time::Piece::_default_locale();
+    my $defaults = [];
+
+    if ($opts) {
+        # Validate and process defaults if provided
+        if ( exists $opts->{defaults} ) {
+            if ( ref( $opts->{defaults} ) eq 'ARRAY' ) {
+                $defaults = $opts->{defaults};
+                unless ( @{ $opts->{defaults} } >= 8 ) {
+                    croak("defaults array must have at least 8 elements!");
+                }
+            } elsif ( ref( $opts->{defaults} ) eq 'HASH' ) {
+
+                ( exists $opts->{defaults}{$_} )
+                  ? push( @{$defaults}, $opts->{defaults}{$_} )
+                  : push( @{$defaults}, undef )
+                  for qw/sec min hour mday mon year wday yday/;
+
+                if ( defined $defaults->[c_year]
+                    && $defaults->[c_year] >= 1000 ) {
+                    $defaults->[c_year] -= 1900;
+                }
+
+            } elsif ( blessed( $opts->{defaults} )
+                && $opts->{defaults}->isa('Time::Piece') ) {
+                # Extract time components from Time::Piece object
+                $defaults = [ @{ $opts->{defaults} }[ c_sec .. c_yday ] ];
+                $islocal  = $opts->{defaults}[c_islocal];
+            } else {
+                croak("defaults must be an array reference, hash reference, or Time::Piece object");
+            }
+        }
+
+        # Check for forced islocal
+        if ( exists $opts->{islocal} && $opts->{islocal} ) {
+            $islocal = 1;
+        }
+    }
+
+    my @vals = _strptime( $string, $format, $islocal, $locales, $defaults );
+
+    return scalar $time->_mktime( \@vals, $islocal );
 }
 
 sub day_list {
@@ -637,7 +728,8 @@ sub cdate {
 
 sub str_compare {
     my ($lhs, $rhs, $reverse) = @_;
-    if (UNIVERSAL::isa($rhs, 'Time::Piece')) {
+
+    if (blessed($rhs) && $rhs->isa('Time::Piece')) {
         $rhs = "$rhs";
     }
     return $reverse ? $rhs cmp $lhs->cdate : $lhs->cdate cmp $rhs;
@@ -650,9 +742,6 @@ use overload
 sub subtract {
     my $time = shift;
     my $rhs = shift;
-    if (UNIVERSAL::isa($rhs, 'Time::Seconds')) {
-        $rhs = $rhs->seconds;
-    }
 
     if (shift)
     {
@@ -665,7 +754,10 @@ sub subtract {
 	return $rhs - "$time";
     }
 
-    if (UNIVERSAL::isa($rhs, 'Time::Piece')) {
+    #TODO: handle math with objects where one is DST and the other isn't
+    #so either convert both to a gmtime object, subtract and then convert to localtime object (would have to add ->to_gmt and ->to_local methods)
+    #or check the tzoffset on each object, if they are different, add in the differing seconds.
+    if (blessed($rhs) && $rhs->isa('Time::Piece')) {
         return Time::Seconds->new($time->epoch - $rhs->epoch);
     }
     else {
@@ -677,10 +769,6 @@ sub subtract {
 sub add {
     my $time = shift;
     my $rhs = shift;
-    if (UNIVERSAL::isa($rhs, 'Time::Seconds')) {
-        $rhs = $rhs->seconds;
-    }
-    croak "Invalid rhs of addition: $rhs" if ref($rhs);
 
     return $time->_mktime(($time->epoch + $rhs), $time->[c_islocal]);
 }
@@ -690,7 +778,7 @@ use overload
 
 sub get_epochs {
     my ($lhs, $rhs, $reverse) = @_;
-    if (!UNIVERSAL::isa($rhs, 'Time::Piece')) {
+    unless (blessed($rhs) && $rhs->isa('Time::Piece')) {
         $rhs = $lhs->new($rhs);
     }
     if ($reverse) {
@@ -760,10 +848,18 @@ sub truncate {
         $time->[c_islocal]);
 }
 
+my $_format_cache = {};
+
 #Given a format and a translate map, replace format flags in
 #accordance with the logic from the translation map subroutines
 sub _translate_format {
     my ( $format, $trans_map, $time ) = @_;
+    my $bad_flags = $IS_WIN32 ? qr/%([eklsVzZ])/ : qr/%([szZ])/;
+    my $can_cache = ($format !~ $bad_flags) ? 1 : 0;
+
+    if ( $can_cache && exists $_format_cache->{$format} ){
+        return $_format_cache->{$format};
+    }
 
     $format =~ s/%%/\e\e/g; #escape the escape
     my $lexer = _build_format_lexer($format);
@@ -774,6 +870,8 @@ sub _translate_format {
 	}
 
     $format =~ s/\e\e/%%/g;
+    $_format_cache->{$_[0]} = $format if $can_cache;
+
     return $format;
 }
 
@@ -795,11 +893,23 @@ sub use_locale {
     #get locale month/day names from posix strftime (from Piece.xs)
     my $locales = _get_localization();
 
-    $locales->{PM} ||= '';
-    $locales->{AM} ||= '';
+    #If AM and PM are the same, set both to ''
+    if (   !$locales->{PM}
+        || !$locales->{AM}
+        || ( $locales->{PM} eq $locales->{AM} ) )
+    {
+        $locales->{PM} = '';
+        $locales->{AM} = '';
+    }
+
+    if (   !$locales->{pm}
+        || !$locales->{am}
+        || ( $locales->{pm} eq $locales->{am} ) )
+    {
+        $locales->{pm} = lc $locales->{PM};
+        $locales->{am} = lc $locales->{AM};
+    }
 
-    $locales->{pm} = lc $locales->{PM};
-    $locales->{am} = lc $locales->{AM};
     #should probably figure out how to get a
     #region specific format for %c someday
     $locales->{c_fmt} = '';
@@ -838,7 +948,7 @@ sub use_locale {
 
 #$Time::Piece::LOCALE is used by strptime and thus needs to be
 #in sync with what ever users change to via day_list() and mon_list().
-#Should probably deprecate this use of gloabl state, but oh well...
+#Should probably deprecate this use of global state, but oh well...
 sub _default_locale {
     my $locales = {};
 
@@ -886,7 +996,7 @@ in perlfunc will still return what you expect.
 
 The module actually implements most of an interface described by
 Larry Wall on the perl5-porters mailing list here:
-L<http://www.xray.mpe.mpg.de/mailing-lists/perl5-porters/2000-01/msg00241.html>
+L<https://www.nntp.perl.org/group/perl.perl5.porters/2000/01/msg5283.html>
 
 =head1 USAGE
 
@@ -964,7 +1074,7 @@ methods.
 
 =head2 Local Locales
 
-Both wdayname (day) and monname (month) allow passing in a list to use
+Both C<wdayname> (day) and C<monname> (month) allow passing in a list to use
 to index the name of the days against. This can be useful if you need
 to implement some form of localisation without actually installing or
 using locales. Note that this is a global override and will affect
@@ -1044,12 +1154,13 @@ Date comparisons are also possible, using the full suite of "<", ">",
 
 =head2 Date Parsing
 
-Time::Piece has a built-in strptime() function (from FreeBSD), allowing
-you incredibly flexible date parsing routines. For example:
+Time::Piece provides flexible date parsing via the built-in strptime() function (from FreeBSD).
+
+=head3 Basic Usage
 
   my $t = Time::Piece->strptime("Sunday 3rd Nov, 1943",
                                 "%A %drd %b, %Y");
-  
+
   print $t->strftime("%a, %d %b %Y");
 
 Outputs:
@@ -1058,8 +1169,72 @@ Outputs:
 
 (see, it's even smart enough to fix my obvious date bug)
 
-For more information see "man strptime", which should be on all unix
-systems.
+=head3 Default Values for Partial Dates
+
+When parsing incomplete date strings, you can provide defaults for missing components:
+
+=head4 Supported Default Types
+
+B<1. Array Reference> - Standard time components (sec, min, hour, mday, mon, year, wday, yday) (see C<perldoc -f localtime>):
+
+  my @defaults = localtime();
+  my $t = Time::Piece->strptime("15 Mar", "%d %b",
+                                { defaults => \@defaults });
+
+B<2. Hash Reference> - Specify only needed components:
+
+  my $t = Time::Piece->strptime("15 Mar", "%d %b",
+                                { defaults => {
+                                    year => 2023,  # Years >= 1000: actual year
+                                    hour => 14,    # Years < 1000: offset from 1900
+                                    min  => 30
+                                } });
+
+Valid keys: C<sec>, C<min>, C<hour>, C<mday>, C<mon>, C<year>, C<wday>, C<yday>, C<isdst>
+
+B<Note:> C<year> in this context doesn't have to be an offset from 1900
+
+B<3. Time::Piece Object> - Copies all components including C<c_islocal>:
+
+  my $base = localtime();
+
+  my $t1 = Time::Piece->strptime("15 Mar", "%d %b",
+                                 { defaults => $base });
+
+  # Shorthand (equivalent)
+  my $t2 = Time::Piece->strptime("15 Mar", "%d %b", $base);
+
+=head4 Format String Defaults
+
+When omitted, format defaults to C<"%a, %d %b %Y %H:%M:%S %Z">:
+
+  # These are equivalent:
+  my $t1 = Time::Piece->strptime($string);
+  my $t2 = Time::Piece->strptime($string, "%a, %d %b %Y %H:%M:%S %Z");
+
+=head3 Timezone Behavior
+
+The returned object's timezone (C<c_islocal>) depends on the calling context:
+
+B<Default: GMT/UTC> (c_islocal = 0)
+
+  Time::Piece->strptime($string, $format)  # Class method returns GMT
+
+B<Local Time> (c_islocal = 1) via:
+
+  # Instance method on localtime object
+  localtime()->strptime($string, $format)
+
+  # Explicit islocal option
+  Time::Piece->strptime($string, $format, { islocal => 1 })
+
+  # Inherited from Time::Piece defaults
+  my $tp_obj = localtime();
+  Time::Piece->strptime($string, $format, $tp_obj)
+
+B<Note:> Parsed values always override defaults. Only missing components use default values.
+
+For more information see "man strptime" on unix systems.
 
 Alternatively look here: L<http://www.unix.com/man-page/FreeBSD/3/strftime/>
 
@@ -1090,22 +1265,6 @@ Returns
 
     ( 'So', 'Mo', 'Di', 'Mi', 'Do', 'Fr', 'Sa' )
 
-=head2 YYYY-MM-DDThh:mm:ss
-
-The ISO 8601 standard defines the date format to be YYYY-MM-DD, and
-the time format to be hh:mm:ss (24 hour clock), and if combined, they
-should be concatenated with date first and with a capital 'T' in front
-of the time.
-
-=head2 Week Number
-
-The I<week number> may be an unknown concept to some readers.  The ISO
-8601 standard defines that weeks begin on a Monday and week 1 of the
-year is the week that includes both January 4th and the first Thursday
-of the year.  In other words, if the first Monday of January is the
-2nd, 3rd, or 4th, the preceding days of the January are part of the
-last week of the preceding year.  Week numbers range from 1 to 53.
-
 =head2 Global Overriding
 
 Finally, it's possible to override localtime and gmtime everywhere, by
@@ -1142,6 +1301,14 @@ moves afoot to fix that in perl. Alternatively use 64 bit perl. Or if none
 of those are options, use the L<DateTime> module which has support for years
 well into the future and past.
 
+Also, the internal representation of Time::Piece->strftime deviates from the
+standard POSIX implementation in that is uses the epoch (instead of separate
+year, month, day parts). This change was added in version 1.30. If you must
+have a more traditional strftime (which will normally never calculate day
+light saving times correctly), you can pass the date parts from Time::Piece
+into the strftime function provided by the POSIX module
+(see strftime in L<POSIX> ).
+
 =head1 AUTHOR
 
 Matt Sergeant, matt@sergeant.org
